@@ -331,7 +331,12 @@ class Dataset(collection.Collection):
         for _ in self.for_each_suffix(field):
             values.append(self[field][idx])
         values = np.hstack(values)
-        return func(values)
+        
+        if values.any():
+            return func(values)
+        else:
+            return np.nan
+
 
     def rms(self, field: str, **filters: Any) -> float:
         """Calculate Root Mean Square of a field"""
@@ -404,13 +409,21 @@ class Dataset(collection.Collection):
 
 
 class Meta(UserDict):
-    def read(self, h5_group):
-        """Read meta data from hdf5-file"""
+    def read(self, h5_group: h5py.Group) -> None:
+        """Read meta data from hdf5-file
+        
+        Args:
+            h5_group:    The hdf5 group that contains the meta data
+        """
         for k, v in h5_group.attrs.items():
             self.data[k] = _h5utils.decode_h5attr(v)
 
-    def write(self, h5_group):
-        """Write meta data to hdf5-file"""
+    def write(self, h5_group: h5py.Group) -> None:
+        """Write meta data to hdf5-file
+
+        Args:
+            h5_group:   The hdf5 group to store the meta data in
+        """
         for k, v in self.items():
             h5_group.attrs[k] = _h5utils.encode_h5attr(v)
 
