@@ -72,11 +72,10 @@ def config_options():
 
 
 @pytest.fixture
-def config_dict():
+def config_dict(gps_dict):
     """A test configuration based on a dictionary"""
-    cfg_dict = gps_dict()
     cfg = config.Configuration("dictionary")
-    cfg.update_from_dict(cfg_dict, section="gps")
+    cfg.update_from_dict(gps_dict, section="gps")
     return cfg
 
 
@@ -87,9 +86,9 @@ def gps_dict():
 
 
 @pytest.fixture
-def config_section():
+def config_section(config_dict):
     """A section with test data"""
-    return config_dict().gps
+    return config_dict.gps
 
 
 #
@@ -112,7 +111,7 @@ def test_read_config_from_file_classmethod(config_file):
 @pytest.mark.skip(reason="as_str() does not print profiles correctly")
 def test_write_config_to_file(config_file, tmpdir):
     """Test that writing a configuration creates a file that is identical to the original"""
-    cfg_path = pathlib.Path(''.join(config_file.sources))
+    cfg_path = pathlib.Path("".join(config_file.sources))
     out_path = pathlib.Path(tmpdir / "test_config.conf")
     config_file.write_to_file(out_path)
 
@@ -135,16 +134,16 @@ def test_read_config_from_options(config_options):
 
 def test_update_config_from_config_section(config_file, config_options):
     """Test that a config section can be copied"""
-    assert "section_1" not in config_file.sections
+    assert "section_1" not in config_file.section_names
     config_file.update_from_config_section(config_options.section_1)
-    assert "section_1" in config_file.sections
+    assert "section_1" in config_file.section_names
     assert str(config_file.section_1) == str(config_options.section_1)
 
 
 def test_update_config_from_options(config_file):
     """Test that a config can be updated from command line options"""
     config_file.master_section = "midgard"
-    sections_before = set(config_file.sections)
+    sections_before = set(config_file.section_names)
     entries_before = set(config_file.midgard.as_list())
     cfg_argv = [
         sys.argv[0],
@@ -161,7 +160,7 @@ def test_update_config_from_options(config_file):
     config_file.update_from_options(allow_new=True)
     sys.argv = remember_sys_argv
 
-    assert set(config_file.sections) - sections_before == {"new_section"}
+    assert set(config_file.section_names) - sections_before == {"new_section"}
     assert set(config_file.midgard.as_list()) - entries_before == {"new_key"}
     assert config_file.midgard.foo.str == "I am an option"
     assert config_file.midgard.pi.str == "4"
@@ -176,9 +175,10 @@ def test_clearing_config(config_file):
 
 
 def test_set_non_existing_master_section(config_file):
-    """Test that setting a non-existing section as master section raises an error"""
+    """Test that setting a non-existing section is ok, but getting from it raises an error"""
+    config_file.master_section = "non_existing"
     with pytest.raises(exceptions.MissingSectionError):
-        config_file.master_section = "non_existing"
+        config_file.non_exisiting
 
 
 def test_access_from_master_section(config_file):
@@ -199,9 +199,16 @@ def test_get_from_master_section_without_master_section(config_file):
         config_file.foo
 
 
+def test_get_from_master_section(config_file):
+    """Test that get can access entries from a master section"""
+    config_file.master_section = "midgard"
+    entry = config_file.get("foo", default="baz")
+    assert entry is config_file.midgard.foo
+
+
 def test_profiles_are_not_separate_sections(config_file):
     """Test that profiles are not registered as separate sections"""
-    assert len([s for s in config_file.sections if s.startswith("profile_test")]) == 1
+    assert len([s for s in config_file.section_names if s.startswith("profile_test")]) == 1
 
 
 def test_profiles_are_prioritized(config_file):
@@ -273,34 +280,63 @@ def test_get_from_configuration(config_file):
     assert entry is config_file.midgard.foo
 
 
-def test_get_from_parent_config(config_file, config_dict):
-    """Test that get can access entries in a parent configuration"""
-    config_dict.parent_config = config_file
+def test_get_from_fallback_config(config_file, config_dict):
+    """Test that get can access entries in a fallback configuration"""
+    config_dict.fallback_config = config_file
     entry = config_dict.get("foo", section="midgard", default="baz")
     assert entry is config_file.midgard.foo
 
 
-def test_get_from_master_section(config_file):
-    """Test that get can access entries from a master section"""
-    config_file.master_section = "midgard"
-    entry = config_file.get("foo", default="baz")
+def test_exists_with_section(config_file):
+    """Test that exists works for both existing and non-existing keys"""
+    assert config_file.exists("foo", section="midgard")
+    assert not config_file.exists("does_not_exist", section="midgard")
+    assert not config_file.exists("foo", section="does_not_exist")
+
+
+def test_exists_with_master_section(config_file):
+    """Test that exists works for both existing and non-existing keys without specifying section"""
+    config_file.master_section = "data_types"
+    assert config_file.exists("str")
+    assert not config_file.exists("does_not_exist")
+
+
+def test_exists_with_master_section_defined(config_file):
+    """Test that exists behaves correctly when master_section is defined and section specified"""
+    config_file.master_section = "data_types"
+    assert config_file.exists("foo", section="midgard")
+    assert not config_file.exists("str", section="str")
+    assert not config_file.exists("foo", section="does_not_exist")
+
+
+def test_getattr_from_fallback_config(config_file, config_dict):
+    """Test that attribute access can get entries in fallback configuration"""
+    config_dict.fallback_config = config_file
+    entry = config_dict.midgard.foo
+    assert entry is config_file.midgard.foo
+
+
+def test_getitem_from_fallback_config(config_file, config_dict):
+    """Test that dictionary access can get entries in fallback configuration"""
+    config_dict.fallback_config = config_file
+    entry = config_dict["midgard"].foo
     assert entry is config_file.midgard.foo
 
 
 def test_add_single_entry(config_file):
     """Test adding a single new entry"""
-    sections_before = set(config_file.sections)
+    sections_before = set(config_file.section_names)
     config_file.update("new_section", "new_key", "new_value", source="test")
-    assert set(config_file.sections) - sections_before == {"new_section"}
+    assert set(config_file.section_names) - sections_before == {"new_section"}
     assert config_file.new_section.new_key.str == "new_value"
     assert config_file.new_section.new_key.source == "test"
 
 
 def test_updating_existing_entry(config_file):
     """Test updating the value of an existing entry"""
-    sections_before = config_file.sections
+    sections_before = config_file.section_names
     config_file.update("midgard", "foo", "new_value", source="test", allow_new=False)
-    assert config_file.sections == sections_before
+    assert config_file.section_names == sections_before
     assert config_file.midgard.foo.str == "new_value"
     assert config_file.midgard.foo.source == "test"
 
@@ -355,22 +391,22 @@ def test_attribute_and_item_access(config_file):
 
 def test_deleting_section_as_item(config_file):
     """Test that deleting a section removes it"""
-    sections_before = set(config_file.sections)
+    sections_before = set(config_file.section_names)
     del config_file["midgard"]
-    assert sections_before - set(config_file.sections) == {"midgard"}
+    assert sections_before - set(config_file.section_names) == {"midgard"}
 
 
 def test_deleting_section_as_attribute(config_file):
     """Test that deleting a section removes it"""
-    sections_before = set(config_file.sections)
+    sections_before = set(config_file.section_names)
     del config_file.midgard
-    assert sections_before - set(config_file.sections) == {"midgard"}
+    assert sections_before - set(config_file.section_names) == {"midgard"}
 
 
 def test_dir_return_sections(config_file):
     """Test that sections are included in dir(configuration)"""
     cfg_dir = dir(config_file)
-    sections = set(config_file.sections)
+    sections = set(config_file.section_names)
     assert len(sections) > 0
     assert set(cfg_dir) & sections == sections
 
